@@ -32,6 +32,7 @@
 
 #include <setjmp.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -45,6 +46,8 @@
 
 
 extern const char* __igt_test_description __attribute__((weak));
+extern bool __igt_plain_output;
+
 
 /**
  * IGT_TEST_DESCRIPTION:
@@ -104,7 +107,7 @@ void __igt_fixture_end(void) __attribute__((noreturn));
  * enumeration (e.g. when enumerating on systems without an intel gpu) such
  * blocks should be annotated with igt_fixture.
  */
-#define igt_fixture for (int igt_tokencat(__tmpint,__LINE__) = 0; \
+#define igt_fixture for (volatile int igt_tokencat(__tmpint,__LINE__) = 0; \
 			 igt_tokencat(__tmpint,__LINE__) < 1 && \
 			 __igt_fixture() && \
 			 (sigsetjmp(igt_subtest_jmpbuf, 1) == 0); \
@@ -119,7 +122,7 @@ struct option;
 #endif
 int igt_subtest_init_parse_opts(int *argc, char **argv,
 				const char *extra_short_opts,
-				struct option *extra_long_opts,
+				const struct option *extra_long_opts,
 				const char *help_str,
 				igt_opt_handler_t extra_opt_handler,
 				void *handler_data);
@@ -194,6 +197,30 @@ bool __igt_run_subtest(const char *subtest_name);
 const char *igt_subtest_name(void);
 bool igt_only_list_subtests(void);
 
+void __igt_subtest_group_save(int *);
+void __igt_subtest_group_restore(int);
+/**
+ * igt_subtest_group:
+ *
+ * Group a set of subtests together with their common setup code
+ *
+ * Testcase with subtests often need to set up a bunch of shared state as the
+ * common test fixture. But if there are multiple with different requirements
+ * the commont setup code can't be extracted, since a test condition failure in
+ * e.g. igt_require() would result in all subsequent tests skipping. Even those
+ * from a different group.
+ *
+ * This macro allows to group together a set of #igt_fixture and #igt_subtest
+ * clauses. If any common setup in a fixture fails, only the subtests in this
+ * group will fail or skip. Subtest groups can be arbitrarily nested.
+ */
+#define igt_subtest_group for (int igt_tokencat(__tmpint,__LINE__) = 0, \
+			       igt_tokencat(__save,__LINE__) = 0; \
+			       igt_tokencat(__tmpint,__LINE__) < 1 && \
+			       (__igt_subtest_group_save(& igt_tokencat(__save,__LINE__) ), true); \
+			       igt_tokencat(__tmpint,__LINE__) ++, \
+			       __igt_subtest_group_restore(igt_tokencat(__save,__LINE__) ))
+
 /**
  * igt_main:
  *
@@ -215,7 +242,7 @@ bool igt_only_list_subtests(void);
 const char *igt_test_name(void);
 void igt_simple_init_parse_opts(int *argc, char **argv,
 				const char *extra_short_opts,
-				struct option *extra_long_opts,
+				const struct option *extra_long_opts,
 				const char *help_str,
 				igt_opt_handler_t extra_opt_handler,
 				void *handler_data);
@@ -453,12 +480,39 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_assert_neq(n1, n2) igt_assert_cmpint(n1, !=, ==, n2)
 
+/**
+ * igt_assert_neq_u32:
+ * @n1: first integer
+ * @n2: second integer
+ *
+ * Like igt_assert_neq(), but for uint32_t.
+ */
+#define igt_assert_neq_u32(n1, n2) igt_assert_cmpuint(n1, !=, ==, n2)
+
+/**
+ * igt_assert_neq_u64:
+ * @n1: first integer
+ * @n2: second integer
+ *
+ * Like igt_assert_neq_u32(), but for uint64_t.
+ */
+#define igt_assert_neq_u64(n1, n2) igt_assert_cmpu64(n1, !=, ==, n2)
+
+/**
+ * igt_assert_neq_double:
+ * @n1: first double
+ * @n2: second double
+ *
+ * Like igt_assert_neq(), but for doubles.
+ */
+#define igt_assert_neq_double(n1, n2) igt_assert_cmpdouble(n1, !=, ==, n2)
+
 /**
  * igt_assert_lte:
  * @n1: first integer
  * @n2: second integer
  *
- * Fails (sub-)test if the second integers is greater than the first.
+ * Fails (sub-)test if the second integer is strictly smaller than the first.
  * Beware that for now this only works on integers.
  *
  * Like igt_assert(), but displays the values being compared on failure instead
@@ -471,7 +525,7 @@ void igt_exit(void) __attribute__((noreturn));
  * @n1: first integer
  * @n2: second integer
  *
- * Fails (sub-)test if the second integers is strictly smaller than the first.
+ * Fails (sub-)test if the second integer is smaller than or equal to the first.
  * Beware that for now this only works on integers.
  *
  * Like igt_assert(), but displays the values being compared on failure instead
@@ -479,6 +533,18 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_assert_lt(n1, n2) igt_assert_cmpint(n1, <, >=, n2)
 
+/**
+ * igt_assert_fd:
+ * @fd: file descriptor
+ *
+ * Fails (sub-) test if the given file descriptor is invalid.
+ *
+ * Like igt_assert(), but displays the values being compared on failure instead
+ * of simply printing the stringified expression.
+ */
+#define igt_assert_fd(fd) \
+	igt_assert_f(fd >= 0, "file descriptor " #fd " failed\n");
+
 /**
  * igt_require:
  * @expr: condition to test
@@ -569,6 +635,7 @@ bool __igt_fork(void);
 	for (int child = 0; child < (num_children); child++) \
 		for (; __igt_fork(); exit(0))
 void igt_waitchildren(void);
+void igt_waitchildren_timeout(int seconds, const char *reason);
 
 /**
  * igt_helper_process:
@@ -625,8 +692,6 @@ typedef void (*igt_exit_handler_t)(int sig);
 
 /* reliable atexit helpers, also work when killed by a signal (if possible) */
 void igt_install_exit_handler(igt_exit_handler_t fn);
-void igt_enable_exit_handler(void);
-void igt_disable_exit_handler(void);
 
 /* helpers to automatically reduce test runtime in simulation */
 bool igt_run_in_simulation(void);
@@ -735,6 +800,40 @@ extern enum igt_log_level igt_log_level;
 void igt_set_timeout(unsigned int seconds,
 		     const char *op);
 
+/**
+ * igt_nsec_elapsed:
+ * @start: measure from this point in time
+ *
+ * Reports the difference in the monotonic clock from the start time
+ * in nanoseconds. On the first invocation, start should be zeroed and will
+ * be set by the call.
+ *
+ * Typical use would be:
+ *
+ * igt_subtest("test") {
+ * 	struct timespec start = {};
+ * 	while (igt_nsec_elapsed(&start) < test_timeout_ns)
+ *	 	do_test();
+ * }
+ *
+ * A handy approximation is to use nsec >> 30 to convert to seconds,
+ * nsec >> 20 to convert to milliseconds - the error is about 8%, acceptable
+ * for test run times.
+ */
+uint64_t igt_nsec_elapsed(struct timespec *start);
+
+/**
+ * igt_seconds_elapsed:
+ * @start: measure from this point in time
+ *
+ * A wrapper around igt_nsec_elapsed that reports the approximate (8% error)
+ * number of seconds since the start point.
+ */
+static inline uint32_t igt_seconds_elapsed(struct timespec *start)
+{
+	return igt_nsec_elapsed(start) >> 30;
+}
+
 void igt_reset_timeout(void);
 
 FILE *__igt_fopen_data(const char* igt_srcdir, const char* igt_datadir,
@@ -743,8 +842,8 @@ FILE *__igt_fopen_data(const char* igt_srcdir, const char* igt_datadir,
  * igt_fopen_data:
  * @filename: filename to open.
  *
- * Open a datafile for test, first try from installation directory
- * then from build directory.
+ * Open a datafile for test, first try from installation directory,
+ * then from build directory, and finally from current directory.
  */
 #define igt_fopen_data(filename) \
 	__igt_fopen_data(IGT_SRCDIR, IGT_DATADIR, filename)
