@@ -145,6 +145,123 @@ const unsigned char* igt_kms_get_base_edid(void)
  *
  * Returns: an alternate edid block
  */
+static const char *igt_plane_prop_names[IGT_NUM_PLANE_PROPS] = {
+	"SRC_X",
+	"SRC_Y",
+	"SRC_W",
+	"SRC_H",
+	"CRTC_X",
+	"CRTC_Y",
+	"CRTC_W",
+	"CRTC_H",
+	"FB_ID",
+	"CRTC_ID",
+	"type",
+	"rotation"
+};
+
+static const char *igt_crtc_prop_names[IGT_NUM_CRTC_PROPS] = {
+	"background_color",
+	"DEGAMMA_LUT",
+	"CTM",
+	"GAMMA_LUT",
+};
+
+static const char *igt_connector_prop_names[IGT_NUM_CONNECTOR_PROPS] = {
+	"scaling mode",
+	"DPMS"
+};
+
+/*
+ * Retrieve all the properies specified in props_name and store them into
+ * plane->atomic_props_plane.
+ */
+static void
+igt_atomic_fill_plane_props(igt_display_t *display, igt_plane_t *plane,
+			int num_props, const char **prop_names)
+{
+	drmModeObjectPropertiesPtr props;
+	int i, j, fd;
+
+	fd = display->drm_fd;
+
+	props = drmModeObjectGetProperties(fd, plane->drm_plane->plane_id, DRM_MODE_OBJECT_PLANE);
+	igt_assert(props);
+
+	for (i = 0; i < props->count_props; i++) {
+		drmModePropertyPtr prop =
+			drmModeGetProperty(fd, props->props[i]);
+
+		for (j = 0; j < num_props; j++) {
+			if (strcmp(prop->name, prop_names[j]) != 0)
+				continue;
+
+			plane->atomic_props_plane[j] = props->props[i];
+			break;
+		}
+
+		drmModeFreeProperty(prop);
+	}
+
+	drmModeFreeObjectProperties(props);
+}
+
+/*
+ * Retrieve all the properies specified in props_name and store them into
+ * config->atomic_props_crtc and config->atomic_props_connector.
+ */
+static void
+igt_atomic_fill_props(igt_display_t *display, igt_output_t *output,
+			int num_crtc_props, const char **crtc_prop_names,
+			int num_connector_props, const char **conn_prop_names)
+{
+	drmModeObjectPropertiesPtr props;
+	int i, j, fd;
+
+	fd = display->drm_fd;
+
+	props = drmModeObjectGetProperties(fd, output->config.crtc->crtc_id, DRM_MODE_OBJECT_CRTC);
+	igt_assert(props);
+
+	for (i = 0; i < props->count_props; i++) {
+		drmModePropertyPtr prop =
+			drmModeGetProperty(fd, props->props[i]);
+
+		for (j = 0; j < num_crtc_props; j++) {
+			if (strcmp(prop->name, crtc_prop_names[j]) != 0)
+				continue;
+
+			output->config.atomic_props_crtc[j] = props->props[i];
+			break;
+		}
+
+		drmModeFreeProperty(prop);
+	}
+
+	drmModeFreeObjectProperties(props);
+	props = NULL;
+	props = drmModeObjectGetProperties(fd, output->config.connector->connector_id, DRM_MODE_OBJECT_CONNECTOR);
+	igt_assert(props);
+
+	for (i = 0; i < props->count_props; i++) {
+		drmModePropertyPtr prop =
+			drmModeGetProperty(fd, props->props[i]);
+
+		for (j = 0; j < num_connector_props; j++) {
+			if (strcmp(prop->name, conn_prop_names[j]) != 0)
+				continue;
+
+			output->config.atomic_props_connector[j] = props->props[i];
+			break;
+		}
+
+		drmModeFreeProperty(prop);
+	}
+
+	drmModeFreeObjectProperties(props);
+
+}
+
 const unsigned char* igt_kms_get_alt_edid(void)
 {
 	update_edid_csum(alt_edid);
@@ -261,20 +378,119 @@ void kmstest_dump_mode(drmModeModeInfo *mode)
  * @fd: DRM fd
  * @crtc_id: DRM CRTC id
  *
- * Returns: The pipe number for the given DRM CRTC @crtc_id. This maps directly
- * to an enum pipe value used in other helper functions.
+ * Returns: The crtc index for the given DRM CRTC ID @crtc_id. The crtc index
+ * is the equivalent of the pipe id.  This value maps directly to an enum pipe
+ * value used in other helper functions.  Returns 0 if the index could not be
+ * determined.
  */
+
 int kmstest_get_pipe_from_crtc_id(int fd, int crtc_id)
 {
-	struct drm_i915_get_pipe_from_crtc_id pfci;
-	int ret;
+	drmModeRes *res;
+	drmModeCrtc *crtc;
+	int i, cur_id;
 
-	memset(&pfci, 0, sizeof(pfci));
-	pfci.crtc_id = crtc_id;
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GET_PIPE_FROM_CRTC_ID, &pfci);
-	igt_assert(ret == 0);
+	res = drmModeGetResources(fd);
+	igt_assert(res);
 
-	return pfci.pipe;
+	for (i = 0; i < res->count_crtcs; i++) {
+		crtc = drmModeGetCrtc(fd, res->crtcs[i]);
+		igt_assert(crtc);
+		cur_id = crtc->crtc_id;
+		drmModeFreeCrtc(crtc);
+		if (cur_id == crtc_id)
+			break;
+	}
+
+	igt_assert(i < res->count_crtcs);
+
+	drmModeFreeResources(res);
+
+	return i;
+}
+
+/**
+ * kmstest_find_crtc_for_connector:
+ * @fd: DRM fd
+ * @res: libdrm resources pointer
+ * @connector: libdrm connector pointer
+ * @crtc_blacklist_idx_mask: a mask of CRTC indexes that we can't return
+ *
+ * Returns: the CRTC ID for a CRTC that fits the connector, otherwise it asserts
+ * false and never returns. The blacklist mask can be used in case you have
+ * CRTCs that are already in use by other connectors.
+ */
+uint32_t kmstest_find_crtc_for_connector(int fd, drmModeRes *res,
+					 drmModeConnector *connector,
+					 uint32_t crtc_blacklist_idx_mask)
+{
+	drmModeEncoder *e;
+	uint32_t possible_crtcs;
+	int i, j;
+
+	for (i = 0; i < connector->count_encoders; i++) {
+		e = drmModeGetEncoder(fd, connector->encoders[i]);
+		possible_crtcs = e->possible_crtcs & ~crtc_blacklist_idx_mask;
+		drmModeFreeEncoder(e);
+
+		for (j = 0; possible_crtcs >> j; j++)
+			if (possible_crtcs & (1 << j))
+				return res->crtcs[j];
+	}
+
+	igt_assert(false);
+}
+
+/**
+ * kmstest_dumb_create:
+ * @fd: open drm file descriptor
+ * @width: width of the buffer in pixels
+ * @height: height of the buffer in pixels
+ * @bpp: bytes per pixel of the buffer
+ *
+ * This wraps the CREATE_DUMB ioctl, which allocates a new dumb buffer object
+ * for the specified dimensions.
+ *
+ * Returns: The file-private handle of the created buffer object
+ */
+uint32_t kmstest_dumb_create(int fd, int width, int height, int bpp,
+			     unsigned *stride, unsigned *size)
+{
+	struct drm_mode_create_dumb create;
+
+	memset(&create, 0, sizeof(create));
+	create.width = width;
+	create.height = height;
+	create.bpp = bpp;
+
+	create.handle = 0;
+	do_ioctl(fd, DRM_IOCTL_MODE_CREATE_DUMB, &create);
+	igt_assert(create.handle);
+	igt_assert(create.size >= width * height * bpp / 8);
+
+	if (stride)
+		*stride = create.pitch;
+
+	if (size)
+		*size = create.size;
+
+	return create.handle;
+}
+
+void *kmstest_dumb_map_buffer(int fd, uint32_t handle, uint64_t size,
+			      unsigned prot)
+{
+	struct drm_mode_map_dumb arg = {};
+	void *ptr;
+
+	arg.handle = handle;
+
+	do_ioctl(fd, DRM_IOCTL_MODE_MAP_DUMB, &arg);
+
+	ptr = mmap(NULL, size, prot, MAP_SHARED, fd, arg.offset);
+	igt_assert(ptr != MAP_FAILED);
+
+	return ptr;
 }
 
 /*
@@ -351,9 +567,7 @@ void kmstest_set_vt_graphics_mode(void)
 
 	igt_install_exit_handler((igt_exit_handler_t) kmstest_restore_vt_mode);
 
-	igt_disable_exit_handler();
 	ret = set_vt_mode(KD_GRAPHICS);
-	igt_enable_exit_handler();
 
 	igt_assert(ret >= 0);
 	orig_vt_mode = ret;
@@ -875,6 +1089,25 @@ void kmstest_unset_all_crtcs(int drm_fd, drmModeResPtr resources)
 	}
 }
 
+/**
+ * kmstest_get_crtc_idx:
+ * @res: the libdrm resources
+ * @crtc_id: the CRTC id
+ *
+ * Get the CRTC index based on its ID. This is useful since a few places of
+ * libdrm deal with CRTC masks.
+ */
+int kmstest_get_crtc_idx(drmModeRes *res, uint32_t crtc_id)
+{
+	int i;
+
+	for (i = 0; i < res->count_crtcs; i++)
+		if (res->crtcs[i] == crtc_id)
+			return i;
+
+	igt_assert(false);
+}
+
 /*
  * A small modeset API
  */
@@ -952,6 +1185,8 @@ static void igt_output_refresh(igt_output_t *output)
 	    kmstest_pipe_name(output->config.pipe));
 
 	display->pipes_in_use |= 1 << output->config.pipe;
+	igt_atomic_fill_props(display, output, IGT_NUM_CRTC_PROPS, igt_crtc_prop_names,
+		IGT_NUM_CONNECTOR_PROPS, igt_connector_prop_names);
 }
 
 static bool
@@ -1021,6 +1256,7 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 	drmModeRes *resources;
 	drmModePlaneRes *plane_resources;
 	int i;
+	int is_atomic = 0;
 
 	memset(display, 0, sizeof(igt_display_t));
 
@@ -1038,6 +1274,7 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 	display->n_pipes = resources->count_crtcs;
 
 	drmSetClientCap(drm_fd, DRM_CLIENT_CAP_UNIVERSAL_PLANES, 1);
+	is_atomic = drmSetClientCap(drm_fd, DRM_CLIENT_CAP_ATOMIC, 1);
 	plane_resources = drmModeGetPlaneResources(display->drm_fd);
 	igt_assert(plane_resources);
 
@@ -1047,6 +1284,7 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 		int p = IGT_PLANE_2;
 		int j, type;
 
+		pipe->crtc_id = resources->crtcs[i];
 		pipe->display = display;
 		pipe->pipe = i;
 
@@ -1094,6 +1332,10 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 
 			plane->pipe = pipe;
 			plane->drm_plane = drm_plane;
+			if (is_atomic == 0) {
+				display->is_atomic = 1;
+				igt_atomic_fill_plane_props(display, plane, IGT_NUM_PLANE_PROPS, igt_plane_prop_names);
+			}
 
 			get_plane_property(display->drm_fd, drm_plane->plane_id,
 					   "rotation",
@@ -1171,6 +1413,7 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 		for (j = 0; j < display->n_pipes; j++) {
 			uint64_t prop_value;
 			igt_pipe_t *pipe = &display->pipes[j];
+
 			if (output->config.crtc) {
 				get_crtc_property(display->drm_fd, output->config.crtc->crtc_id,
 						   "background_color",
@@ -1178,6 +1421,21 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 						   &prop_value,
 						   NULL);
 				pipe->background = (uint32_t)prop_value;
+				get_crtc_property(display->drm_fd, output->config.crtc->crtc_id,
+						  "DEGAMMA_LUT",
+						  &pipe->degamma_property,
+						  NULL,
+						  NULL);
+				get_crtc_property(display->drm_fd, output->config.crtc->crtc_id,
+						  "CTM",
+						  &pipe->ctm_property,
+						  NULL,
+						  NULL);
+				get_crtc_property(display->drm_fd, output->config.crtc->crtc_id,
+						  "GAMMA_LUT",
+						  &pipe->gamma_property,
+						  NULL,
+						  NULL);
 			}
 		}
 	}
@@ -1327,6 +1585,16 @@ static igt_plane_t *igt_pipe_get_plane(igt_pipe_t *pipe, enum igt_plane plane)
 	return &pipe->planes[idx];
 }
 
+bool igt_pipe_get_property(igt_pipe_t *pipe, const char *name,
+			   uint32_t *prop_id, uint64_t *value,
+			   drmModePropertyPtr *prop)
+{
+	return get_crtc_property(pipe->display->drm_fd,
+				 pipe->crtc_id,
+				 name,
+				 prop_id, value, prop);
+}
+
 static uint32_t igt_plane_get_fb_id(igt_plane_t *plane)
 {
 	if (plane->fb)
@@ -1349,6 +1617,80 @@ static uint32_t igt_plane_get_fb_gem_handle(igt_plane_t *plane)
 	igt_assert(r == 0);	\
 }
 
+
+
+
+/*
+ * Add position and fb changes of a plane to the atomic property set
+ */
+static void
+igt_atomic_prepare_plane_commit(igt_plane_t *plane, igt_output_t *output,
+	drmModeAtomicReq *req)
+{
+
+	igt_display_t *display = output->display;
+	uint32_t fb_id, crtc_id;
+
+	igt_assert(plane->drm_plane);
+
+	/* it's an error to try an unsupported feature */
+	igt_assert(igt_plane_supports_rotation(plane) ||
+			!plane->rotation_changed);
+
+	fb_id = igt_plane_get_fb_id(plane);
+	crtc_id = output->config.crtc->crtc_id;
+
+	LOG(display,
+	    "%s: populating plane data: %s.%d, fb %u\n",
+	    igt_output_name(output),
+	    kmstest_pipe_name(output->config.pipe),
+	    plane->index,
+	    fb_id);
+
+	if (plane->fb_changed) {
+		igt_atomic_populate_plane_req(req, plane, IGT_PLANE_CRTC_ID, fb_id ? crtc_id : 0);
+		igt_atomic_populate_plane_req(req, plane, IGT_PLANE_FB_ID, fb_id);
+	}
+
+	if (plane->position_changed || plane->size_changed) {
+		uint32_t src_x = IGT_FIXED(plane->src_x, 0); /* src_x */
+		uint32_t src_y = IGT_FIXED(plane->src_y, 0); /* src_y */
+		uint32_t src_w = IGT_FIXED(plane->src_w, 0); /* src_w */
+		uint32_t src_h = IGT_FIXED(plane->src_h, 0); /* src_h */
+		int32_t crtc_x = plane->crtc_x;
+		int32_t crtc_y = plane->crtc_y;
+		uint32_t crtc_w = plane->crtc_w;
+		uint32_t crtc_h = plane->crtc_h;
+
+		LOG(display,
+		"src = (%d, %d) %u x %u "
+		"dst = (%d, %d) %u x %u\n",
+		src_x >> 16, src_y >> 16, src_w >> 16, src_h >> 16,
+		crtc_x, crtc_y, crtc_w, crtc_h);
+
+		igt_atomic_populate_plane_req(req, plane, IGT_PLANE_SRC_X, src_x);
+		igt_atomic_populate_plane_req(req, plane, IGT_PLANE_SRC_Y, src_y);
+		igt_atomic_populate_plane_req(req, plane, IGT_PLANE_SRC_W, src_w);
+		igt_atomic_populate_plane_req(req, plane, IGT_PLANE_SRC_H, src_h);
+
+		igt_atomic_populate_plane_req(req, plane, IGT_PLANE_CRTC_X, crtc_x);
+		igt_atomic_populate_plane_req(req, plane, IGT_PLANE_CRTC_Y, crtc_y);
+		igt_atomic_populate_plane_req(req, plane, IGT_PLANE_CRTC_W, crtc_w);
+		igt_atomic_populate_plane_req(req, plane, IGT_PLANE_CRTC_H, crtc_h);
+	}
+
+	if (plane->rotation_changed)
+		igt_atomic_populate_plane_req(req, plane,
+			IGT_PLANE_ROTATION, plane->rotation);
+
+	plane->fb_changed = false;
+	plane->position_changed = false;
+	plane->size_changed = false;
+	plane->rotation_changed = false;
+}
+
+
+
 /*
  * Commit position and fb changes to a DRM plane via the SetPlane ioctl; if the
  * DRM call to program the plane fails, we'll either fail immediately (for
@@ -1402,10 +1744,10 @@ static int igt_drm_plane_commit(igt_plane_t *plane,
 		CHECK_RETURN(ret, fail_on_error);
 	} else if (plane->fb_changed || plane->position_changed ||
 		plane->size_changed) {
-		src_x = IGT_FIXED(plane->fb->src_x,0); /* src_x */
-		src_y = IGT_FIXED(plane->fb->src_y,0); /* src_y */
-		src_w = IGT_FIXED(plane->fb->src_w,0); /* src_w */
-		src_h = IGT_FIXED(plane->fb->src_h,0); /* src_h */
+		src_x = IGT_FIXED(plane->src_x,0); /* src_x */
+		src_y = IGT_FIXED(plane->src_y,0); /* src_y */
+		src_w = IGT_FIXED(plane->src_w,0); /* src_w */
+		src_h = IGT_FIXED(plane->src_h,0); /* src_h */
 		crtc_x = plane->crtc_x;
 		crtc_y = plane->crtc_y;
 		crtc_w = plane->crtc_w;
@@ -1601,7 +1943,7 @@ static int igt_plane_commit(igt_plane_t *plane,
 		return igt_primary_plane_commit_legacy(plane, output,
 						       fail_on_error);
 	} else {
-		return igt_drm_plane_commit(plane, output, fail_on_error);
+			return igt_drm_plane_commit(plane, output, fail_on_error);
 	}
 }
 
@@ -1634,6 +1976,17 @@ static int igt_output_commit(igt_output_t *output,
 		pipe->background_changed = false;
 	}
 
+	if (pipe->color_mgmt_changed) {
+		igt_crtc_set_property(output, pipe->degamma_property,
+				      pipe->degamma_blob);
+		igt_crtc_set_property(output, pipe->ctm_property,
+				      pipe->ctm_blob);
+		igt_crtc_set_property(output, pipe->gamma_property,
+				      pipe->gamma_blob);
+
+		pipe->color_mgmt_changed = false;
+	}
+
 	for (i = 0; i < pipe->n_planes; i++) {
 		igt_plane_t *plane = &pipe->planes[i];
 
@@ -1655,6 +2008,94 @@ static int igt_output_commit(igt_output_t *output,
 	return 0;
 }
 
+
+/*
+ * Add crtc property changes to the atomic property set
+ */
+static void igt_atomic_prepare_crtc_commit(igt_output_t *output, drmModeAtomicReq *req)
+{
+
+	igt_pipe_t *pipe_obj = igt_output_get_driving_pipe(output);
+
+	if (pipe_obj->background_changed)
+		igt_atomic_populate_crtc_req(req, output, IGT_CRTC_BACKGROUND, pipe_obj->background);
+
+	if (pipe_obj->color_mgmt_changed) {
+		igt_atomic_populate_crtc_req(req, output, IGT_CRTC_DEGAMMA_LUT, pipe_obj->degamma_blob);
+		igt_atomic_populate_crtc_req(req, output, IGT_CRTC_CTM, pipe_obj->ctm_blob);
+		igt_atomic_populate_crtc_req(req, output, IGT_CRTC_GAMMA_LUT, pipe_obj->gamma_blob);
+	}
+
+	/*
+	 *	TODO: Add all crtc level properties here
+	 */
+}
+
+/*
+ * Add connector property changes to the atomic property set
+ */
+static void igt_atomic_prepare_connector_commit(igt_output_t *output, drmModeAtomicReq *req)
+{
+
+	struct kmstest_connector_config *config = &output->config;
+
+	if (config->connector_scaling_mode_changed)
+		igt_atomic_populate_connector_req(req, output, IGT_CONNECTOR_SCALING_MODE, config->connector_scaling_mode);
+
+	if (config->connector_dpms_changed)
+		igt_atomic_populate_connector_req(req, output, IGT_CONNECTOR_DPMS, config->connector_dpms);
+	/*
+	 *	TODO: Add all other connector level properties here
+	 */
+
+}
+
+/*
+ * Commit all the changes of all the planes,crtcs, connectors
+ * atomically using drmModeAtomicCommit()
+ */
+static int igt_atomic_commit(igt_display_t *display)
+{
+
+	int ret = 0;
+	drmModeAtomicReq *req;
+	igt_output_t *output;
+	if (display->is_atomic != 1)
+		return -1;
+	req = drmModeAtomicAlloc();
+	drmModeAtomicSetCursor(req, 0);
+
+	for_each_connected_output(display, output) {
+		igt_pipe_t *pipe_obj;
+		igt_plane_t *plane;
+		enum pipe pipe;
+
+		/*
+		 * Add CRTC Properties to the property set
+		 */
+		igt_atomic_prepare_crtc_commit(output, req);
+
+		/*
+		 * Add Connector Properties to the property set
+		 */
+		igt_atomic_prepare_connector_commit(output, req);
+
+
+		pipe_obj = igt_output_get_driving_pipe(output);
+
+		pipe = pipe_obj->pipe;
+
+		for_each_plane_on_pipe(display, pipe, plane) {
+			igt_atomic_prepare_plane_commit(plane, output, req);
+		}
+
+	}
+
+	ret = drmModeAtomicCommit(display->drm_fd, req, 0, NULL);
+	drmModeAtomicFree(req);
+	return ret;
+
+}
 /*
  * Commit all plane changes across all outputs of the display.
  *
@@ -1669,23 +2110,36 @@ static int do_display_commit(igt_display_t *display,
 			     bool fail_on_error)
 {
 	int i, ret;
+	int valid_outs = 0;
 
 	LOG_INDENT(display, "commit");
 
 	igt_display_refresh(display);
 
+	if (s == COMMIT_ATOMIC) {
+
+		ret = igt_atomic_commit(display);
+		CHECK_RETURN(ret, fail_on_error);
+		return 0;
+
+	}
+
 	for (i = 0; i < display->n_outputs; i++) {
 		igt_output_t *output = &display->outputs[i];
 
 		if (!output->valid)
 			continue;
 
+		valid_outs++;
 		ret = igt_output_commit(output, s, fail_on_error);
 		CHECK_RETURN(ret, fail_on_error);
 	}
 
 	LOG_UNINDENT(display);
 
+	if (valid_outs == 0)
+		return -1;
+
 	igt_debug_wait_for_keypress("modeset");
 
 	return 0;
@@ -1819,10 +2273,10 @@ void igt_plane_set_fb(igt_plane_t *plane, struct igt_fb *fb)
 		plane->crtc_h = fb->height;
 
 		/* set default src pos/size as fb size */
-		fb->src_x = 0;
-		fb->src_y = 0;
-		fb->src_w = fb->width;
-		fb->src_h = fb->height;
+		plane->src_x = 0;
+		plane->src_y = 0;
+		plane->src_w = fb->width;
+		plane->src_h = fb->height;
 	} else {
 		plane->crtc_w = 0;
 		plane->crtc_h = 0;
@@ -1889,8 +2343,8 @@ void igt_fb_set_position(struct igt_fb *fb, igt_plane_t *plane,
 	LOG(display, "%s.%d: fb_set_position(%d,%d)\n",
 	    kmstest_pipe_name(pipe->pipe), plane->index, x, y);
 
-	fb->src_x = x;
-	fb->src_y = y;
+	plane->src_x = x;
+	plane->src_y = y;
 
 	plane->fb_changed = true;
 }
@@ -1912,11 +2366,11 @@ void igt_fb_set_size(struct igt_fb *fb, igt_plane_t *plane,
 	igt_pipe_t *pipe = plane->pipe;
 	igt_display_t *display = pipe->display;
 
-	LOG(display, "%s.%d: fb_set_position(%dx%d)\n",
+	LOG(display, "%s.%d: fb_set_size(%dx%d)\n",
 	    kmstest_pipe_name(pipe->pipe), plane->index, w, h);
 
-	fb->src_w = w;
-	fb->src_h = h;
+	plane->src_w = w;
+	plane->src_h = h;
 
 	plane->fb_changed = true;
 }
@@ -1966,6 +2420,44 @@ void igt_plane_set_rotation(igt_plane_t *plane, igt_rotation_t rotation)
 	plane->rotation_changed = true;
 }
 
+static void
+igt_pipe_replace_blob(igt_pipe_t *pipe, uint64_t *blob, void *ptr, size_t length)
+{
+	igt_display_t *display = pipe->display;
+	uint32_t blob_id = 0;
+
+	if (*blob != 0)
+		igt_assert(drmModeDestroyPropertyBlob(display->drm_fd,
+						      *blob) == 0);
+
+	if (length > 0)
+		igt_assert(drmModeCreatePropertyBlob(display->drm_fd,
+						     ptr, length, &blob_id) == 0);
+
+	*blob = blob_id;
+}
+
+void
+igt_pipe_set_degamma_lut(igt_pipe_t *pipe, void *ptr, size_t length)
+{
+	igt_pipe_replace_blob(pipe, &pipe->degamma_blob, ptr, length);
+	pipe->color_mgmt_changed = 1;
+}
+
+void
+igt_pipe_set_ctm_matrix(igt_pipe_t *pipe, void *ptr, size_t length)
+{
+	igt_pipe_replace_blob(pipe, &pipe->ctm_blob, ptr, length);
+	pipe->color_mgmt_changed = 1;
+}
+
+void
+igt_pipe_set_gamma_lut(igt_pipe_t *pipe, void *ptr, size_t length)
+{
+	igt_pipe_replace_blob(pipe, &pipe->gamma_blob, ptr, length);
+	pipe->color_mgmt_changed = 1;
+}
+
 /**
  * igt_crtc_set_background:
  * @pipe: pipe pointer to which background color to be set
@@ -2018,7 +2510,7 @@ void igt_enable_connectors(void)
 	drmModeConnector *c;
 	int drm_fd;
 
-	drm_fd = drm_open_driver(DRIVER_INTEL);
+	drm_fd = drm_open_driver(DRIVER_ANY);
 
 	res = drmModeGetResources(drm_fd);
 
