@@ -77,6 +77,9 @@
  * distinguish them.
  */
 
+int (*igt_ioctl)(int fd, unsigned long request, void *arg) = drmIoctl;
+
+
 /**
  * gem_handle_to_libdrm_bo:
  * @bufmgr: libdrm buffer manager instance
@@ -126,7 +129,7 @@ gem_get_tiling(int fd, uint32_t handle, uint32_t *tiling, uint32_t *swizzle)
 	memset(&get_tiling, 0, sizeof(get_tiling));
 	get_tiling.handle = handle;
 
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_GET_TILING, &get_tiling);
+	ret = igt_ioctl(fd, DRM_IOCTL_I915_GEM_GET_TILING, &get_tiling);
 	igt_assert(ret == 0);
 
 	*tiling = get_tiling.tiling_mode;
@@ -138,6 +141,8 @@ int __gem_set_tiling(int fd, uint32_t handle, uint32_t tiling, uint32_t stride)
 	struct drm_i915_gem_set_tiling st;
 	int ret;
 
+	igt_require_intel(fd);
+
 	memset(&st, 0, sizeof(st));
 	do {
 		st.handle = handle;
@@ -180,6 +185,23 @@ struct local_drm_i915_gem_caching {
 #define LOCAL_DRM_IOCTL_I915_GEM_GET_CACHEING \
 	DRM_IOWR(DRM_COMMAND_BASE + LOCAL_DRM_I915_GEM_GET_CACHEING, struct local_drm_i915_gem_caching)
 
+static int __gem_set_caching(int fd, uint32_t handle, uint32_t caching)
+{
+	struct local_drm_i915_gem_caching arg;
+	int err;
+
+	memset(&arg, 0, sizeof(arg));
+	arg.handle = handle;
+	arg.caching = caching;
+
+	err = 0;
+	if (igt_ioctl(fd, LOCAL_DRM_IOCTL_I915_GEM_SET_CACHEING, &arg)) {
+		err = -errno;
+		igt_assert(errno == ENOTTY || errno == EINVAL);
+	}
+	return err;
+}
+
 /**
  * gem_set_caching:
  * @fd: open i915 drm file descriptor
@@ -193,17 +215,7 @@ struct local_drm_i915_gem_caching {
  */
 void gem_set_caching(int fd, uint32_t handle, uint32_t caching)
 {
-	struct local_drm_i915_gem_caching arg;
-	int ret;
-
-	memset(&arg, 0, sizeof(arg));
-	arg.handle = handle;
-	arg.caching = caching;
-
-	ret = drmIoctl(fd, LOCAL_DRM_IOCTL_I915_GEM_SET_CACHEING, &arg);
-	igt_assert(ret == 0 || (errno == ENOTTY || errno == EINVAL));
-
-	igt_require(ret == 0);
+	igt_require(__gem_set_caching(fd, handle, caching) == 0);
 	errno = 0;
 }
 
@@ -221,8 +233,8 @@ uint32_t gem_get_caching(int fd, uint32_t handle)
 	struct local_drm_i915_gem_caching arg;
 	int ret;
 
+	memset(&arg, 0, sizeof(arg));
 	arg.handle = handle;
-	arg.caching = 0;
 	ret = ioctl(fd, LOCAL_DRM_IOCTL_I915_GEM_GET_CACHEING, &arg);
 	igt_assert(ret == 0);
 	errno = 0;
@@ -291,11 +303,30 @@ void gem_close(int fd, uint32_t handle)
 {
 	struct drm_gem_close close_bo;
 
+	igt_assert_neq(handle, 0);
+
 	memset(&close_bo, 0, sizeof(close_bo));
 	close_bo.handle = handle;
 	do_ioctl(fd, DRM_IOCTL_GEM_CLOSE, &close_bo);
 }
 
+static int __gem_write(int fd, uint32_t handle, uint64_t offset, const void *buf, uint64_t length)
+{
+	struct drm_i915_gem_pwrite gem_pwrite;
+	int err;
+
+	memset(&gem_pwrite, 0, sizeof(gem_pwrite));
+	gem_pwrite.handle = handle;
+	gem_pwrite.offset = offset;
+	gem_pwrite.size = length;
+	gem_pwrite.data_ptr = (uintptr_t)buf;
+
+	err = 0;
+	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_PWRITE, &gem_pwrite))
+		err = -errno;
+	return err;
+}
+
 /**
  * gem_write:
  * @fd: open i915 drm file descriptor
@@ -309,16 +340,25 @@ void gem_close(int fd, uint32_t handle)
  */
 void gem_write(int fd, uint32_t handle, uint64_t offset, const void *buf, uint64_t length)
 {
-	struct drm_i915_gem_pwrite gem_pwrite;
-
-	memset(&gem_pwrite, 0, sizeof(gem_pwrite));
-	gem_pwrite.handle = handle;
-	gem_pwrite.offset = offset;
-	gem_pwrite.size = length;
-	gem_pwrite.data_ptr = (uintptr_t)buf;
-	do_ioctl(fd, DRM_IOCTL_I915_GEM_PWRITE, &gem_pwrite);
+	igt_assert_eq(__gem_write(fd, handle, offset, buf, length), 0);
 }
 
+static int __gem_read(int fd, uint32_t handle, uint64_t offset, void *buf, uint64_t length)
+{
+	struct drm_i915_gem_pread gem_pread;
+	int err;
+
+	memset(&gem_pread, 0, sizeof(gem_pread));
+	gem_pread.handle = handle;
+	gem_pread.offset = offset;
+	gem_pread.size = length;
+	gem_pread.data_ptr = (uintptr_t)buf;
+
+	err = 0;
+	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_PREAD, &gem_pread))
+		err = -errno;
+	return err;
+}
 /**
  * gem_read:
  * @fd: open i915 drm file descriptor
@@ -332,14 +372,7 @@ void gem_write(int fd, uint32_t handle, uint64_t offset, const void *buf, uint64
  */
 void gem_read(int fd, uint32_t handle, uint64_t offset, void *buf, uint64_t length)
 {
-	struct drm_i915_gem_pread gem_pread;
-
-	memset(&gem_pread, 0, sizeof(gem_pread));
-	gem_pread.handle = handle;
-	gem_pread.offset = offset;
-	gem_pread.size = length;
-	gem_pread.data_ptr = (uintptr_t)buf;
-	do_ioctl(fd, DRM_IOCTL_I915_GEM_PREAD, &gem_pread);
+	igt_assert_eq(__gem_read(fd, handle, offset, buf, length), 0);
 }
 
 /**
@@ -367,6 +400,36 @@ void gem_set_domain(int fd, uint32_t handle,
 	do_ioctl(fd, DRM_IOCTL_I915_GEM_SET_DOMAIN, &set_domain);
 }
 
+/**
+ * __gem_wait:
+ * @fd: open i915 drm file descriptor
+ * @handle: gem buffer object handle
+ * @timeout_ns: [in] time to wait, [out] remaining time (in nanoseconds)
+ *
+ * This functions waits for outstanding rendering to complete, upto
+ * the timeout_ns. If no timeout_ns is provided, the wait is indefinite and
+ * only returns upon an error or when the rendering is complete.
+ */
+int gem_wait(int fd, uint32_t handle, int64_t *timeout_ns)
+{
+	struct drm_i915_gem_wait wait;
+	int ret;
+
+	memset(&wait, 0, sizeof(wait));
+	wait.bo_handle = handle;
+	wait.timeout_ns = timeout_ns ? *timeout_ns : -1;
+	wait.flags = 0;
+
+	ret = 0;
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_WAIT, &wait))
+		ret = -errno;
+
+	if (timeout_ns)
+		*timeout_ns = wait.timeout_ns;
+
+	return ret;
+}
+
 /**
  * gem_sync:
  * @fd: open i915 drm file descriptor
@@ -376,20 +439,86 @@ void gem_set_domain(int fd, uint32_t handle,
  */
 void gem_sync(int fd, uint32_t handle)
 {
-	struct drm_i915_gem_wait wait;
+	if (gem_wait(fd, handle, NULL))
+		gem_set_domain(fd, handle,
+			       I915_GEM_DOMAIN_GTT,
+			       I915_GEM_DOMAIN_GTT);
+	errno = 0;
+}
 
-	memset(&wait, 0, sizeof(wait));
-	wait.bo_handle = handle;
-	wait.timeout_ns =-1;
-	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_WAIT, &wait) == 0) {
-		errno = 0;
-		return;
+
+bool gem_create__has_stolen_support(int fd)
+{
+	static int has_stolen_support = -1;
+	struct drm_i915_getparam gp;
+	int val = -1;
+
+	if (has_stolen_support < 0) {
+		memset(&gp, 0, sizeof(gp));
+		gp.param = 36; /* CREATE_VERSION */
+		gp.value = &val;
+
+		/* Do we have the extended gem_create_ioctl? */
+		ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp);
+		has_stolen_support = val >= 2;
 	}
 
-	gem_set_domain(fd, handle,
-		       I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+	return has_stolen_support;
 }
 
+struct local_i915_gem_create_v2 {
+	uint64_t size;
+	uint32_t handle;
+	uint32_t pad;
+#define I915_CREATE_PLACEMENT_STOLEN (1<<0)
+	uint32_t flags;
+};
+
+#define LOCAL_IOCTL_I915_GEM_CREATE       DRM_IOWR(DRM_COMMAND_BASE + DRM_I915_GEM_CREATE, struct local_i915_gem_create_v2)
+uint32_t __gem_create_stolen(int fd, uint64_t size)
+{
+	struct local_i915_gem_create_v2 create;
+	int ret;
+
+	memset(&create, 0, sizeof(create));
+	create.handle = 0;
+	create.size = size;
+	create.flags = I915_CREATE_PLACEMENT_STOLEN;
+	ret = igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_CREATE, &create);
+
+	if (ret < 0)
+		return 0;
+
+	errno = 0;
+	return create.handle;
+}
+
+/**
+ * gem_create_stolen:
+ * @fd: open i915 drm file descriptor
+ * @size: desired size of the buffer
+ *
+ * This wraps the new GEM_CREATE ioctl, which allocates a new gem buffer
+ * object of @size and placement in stolen memory region.
+ *
+ * Returns: The file-private handle of the created buffer object
+ */
+
+uint32_t gem_create_stolen(int fd, uint64_t size)
+{
+	struct local_i915_gem_create_v2 create;
+
+	memset(&create, 0, sizeof(create));
+	create.handle = 0;
+	create.size = size;
+	create.flags = I915_CREATE_PLACEMENT_STOLEN;
+	do_ioctl(fd, LOCAL_IOCTL_I915_GEM_CREATE, &create);
+	igt_assert(create.handle);
+
+	return create.handle;
+}
+
+
 uint32_t __gem_create(int fd, int size)
 {
 	struct drm_i915_gem_create create;
@@ -398,7 +527,7 @@ uint32_t __gem_create(int fd, int size)
 	memset(&create, 0, sizeof(create));
 	create.handle = 0;
 	create.size = size;
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create);
+	ret = igt_ioctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create);
 
 	if (ret < 0)
 		return 0;
@@ -430,6 +559,23 @@ uint32_t gem_create(int fd, uint64_t size)
 	return create.handle;
 }
 
+/**
+ * __gem_execbuf:
+ * @fd: open i915 drm file descriptor
+ * @execbuf: execbuffer data structure
+ *
+ * This wraps the EXECBUFFER2 ioctl, which submits a batchbuffer for the gpu to
+ * run. This is allowed to fail, with -errno returned.
+ */
+int __gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
+{
+	int err = 0;
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, execbuf))
+		err = -errno;
+	errno = 0;
+	return err;
+}
+
 /**
  * gem_execbuf:
  * @fd: open i915 drm file descriptor
@@ -440,11 +586,7 @@ uint32_t gem_create(int fd, uint64_t size)
  */
 void gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
 {
-	int result;
-
-	result = drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, execbuf);
-	igt_assert(result == 0);
-	errno = 0;
+	igt_assert_eq(__gem_execbuf(fd, execbuf), 0);
 }
 
 /**
@@ -466,7 +608,7 @@ void *__gem_mmap__gtt(int fd, uint32_t handle, uint64_t size, unsigned prot)
 
 	memset(&mmap_arg, 0, sizeof(mmap_arg));
 	mmap_arg.handle = handle;
-	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_MMAP_GTT, &mmap_arg))
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_MMAP_GTT, &mmap_arg))
 		return NULL;
 
 	ptr = mmap64(0, size, prot, MAP_SHARED, fd, mmap_arg.offset);
@@ -532,7 +674,7 @@ bool gem_mmap__has_wc(int fd)
 			arg.offset = 0;
 			arg.size = 4096;
 			arg.flags = I915_MMAP_WC;
-			has_wc = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_MMAP_v2, &arg) == 0;
+			has_wc = igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_MMAP_v2, &arg) == 0;
 			gem_close(fd, arg.handle);
 		}
 		errno = 0;
@@ -570,7 +712,7 @@ void *__gem_mmap__wc(int fd, uint32_t handle, uint64_t offset, uint64_t size, un
 	arg.offset = offset;
 	arg.size = size;
 	arg.flags = I915_MMAP_WC;
-	if (drmIoctl(fd, LOCAL_IOCTL_I915_GEM_MMAP_v2, &arg))
+	if (igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_MMAP_v2, &arg))
 		return NULL;
 
 	errno = 0;
@@ -617,7 +759,7 @@ void *__gem_mmap__cpu(int fd, uint32_t handle, uint64_t offset, uint64_t size, u
 	mmap_arg.handle = handle;
 	mmap_arg.offset = offset;
 	mmap_arg.size = size;
-	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_MMAP, &mmap_arg))
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_MMAP, &mmap_arg))
 		return NULL;
 
 	errno = 0;
@@ -684,12 +826,14 @@ int gem_madvise(int fd, uint32_t handle, int state)
 uint32_t gem_context_create(int fd)
 {
 	struct drm_i915_gem_context_create create;
-	int ret;
 
 	memset(&create, 0, sizeof(create));
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_CREATE, &create);
-	igt_require(ret == 0 || (errno != ENODEV && errno != EINVAL));
-	igt_assert(ret == 0);
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_CREATE, &create)) {
+		int err = -errno;
+		igt_skip_on(err == -ENODEV || errno == -EINVAL);
+		igt_assert_eq(err, 0);
+	}
+	igt_assert(create.ctx_id != 0);
 	errno = 0;
 
 	return create.ctx_id;
@@ -703,7 +847,7 @@ int __gem_context_destroy(int fd, uint32_t ctx_id)
 	memset(&destroy, 0, sizeof(destroy));
 	destroy.ctx_id = ctx_id;
 
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_DESTROY, &destroy);
+	ret = igt_ioctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_DESTROY, &destroy);
 	if (ret)
 		return -errno;
 	return 0;
@@ -744,6 +888,16 @@ void gem_context_get_param(int fd, struct local_i915_gem_context_param *p)
 	do_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, p);
 }
 
+int __gem_context_set_param(int fd, struct local_i915_gem_context_param *p)
+{
+#define LOCAL_I915_GEM_CONTEXT_SETPARAM       0x35
+#define LOCAL_IOCTL_I915_GEM_CONTEXT_SETPARAM DRM_IOWR (DRM_COMMAND_BASE + LOCAL_I915_GEM_CONTEXT_SETPARAM, struct local_i915_gem_context_param)
+	if (igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_SETPARAM, p))
+		return -errno;
+
+	errno = 0;
+	return 0;
+}
 /**
  * gem_context_set_param:
  * @fd: open i915 drm file descriptor
@@ -756,9 +910,7 @@ void gem_context_get_param(int fd, struct local_i915_gem_context_param *p)
  */
 void gem_context_set_param(int fd, struct local_i915_gem_context_param *p)
 {
-#define LOCAL_I915_GEM_CONTEXT_SETPARAM       0x35
-#define LOCAL_IOCTL_I915_GEM_CONTEXT_SETPARAM DRM_IOWR (DRM_COMMAND_BASE + LOCAL_I915_GEM_CONTEXT_SETPARAM, struct local_i915_gem_context_param)
-	do_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_SETPARAM, p);
+	igt_assert(__gem_context_set_param(fd, p) == 0);
 }
 
 /**
@@ -778,7 +930,7 @@ void gem_context_require_param(int fd, uint64_t param)
 	p.value = 0;
 	p.size = 0;
 
-	igt_require(drmIoctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0);
+	igt_require(igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0);
 }
 
 void gem_context_require_ban_period(int fd)
@@ -793,12 +945,53 @@ void gem_context_require_ban_period(int fd)
 		p.value = 0;
 		p.size = 0;
 
-		has_ban_period = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0;
+		has_ban_period = igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0;
 	}
 
 	igt_require(has_ban_period);
 }
 
+int __gem_userptr(int fd, void *ptr, int size, int read_only, uint32_t flags, uint32_t *handle)
+{
+	struct local_i915_gem_userptr userptr;
+	int ret;
+
+	memset(&userptr, 0, sizeof(userptr));
+	userptr.user_ptr = (uintptr_t)ptr;
+	userptr.user_size = size;
+	userptr.flags = flags;
+	if (read_only)
+		userptr.flags |= LOCAL_I915_USERPTR_READ_ONLY;
+
+	ret = igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr);
+	if (ret)
+		ret = errno;
+	igt_skip_on_f(ret == ENODEV &&
+			(flags & LOCAL_I915_USERPTR_UNSYNCHRONIZED) == 0 &&
+			!read_only,
+			"Skipping, synchronized mappings with no kernel CONFIG_MMU_NOTIFIER?");
+	if (ret == 0)
+		*handle = userptr.handle;
+
+	return ret;
+}
+
+/**
+ * gem_userptr:
+ * @fd: open i915 drm file descriptor
+ * @ptr: userptr pointer to be passed
+ * @size: desired size of the buffer
+ * @read_only: specify whether userptr is opened read only
+ * @flags: other userptr flags
+ * @handle: returned handle for the object
+ *
+ * Returns userptr handle for the GEM object.
+ */
+void gem_userptr(int fd, void *ptr, int size, int read_only, uint32_t flags, uint32_t *handle)
+{
+	igt_assert_eq(__gem_userptr(fd, ptr, size, read_only, flags, handle), 0);
+}
+
 /**
  * gem_sw_finish:
  * @fd: open i915 drm file descriptor
@@ -844,18 +1037,18 @@ bool gem_bo_busy(int fd, uint32_t handle)
 /* feature test helpers */
 
 /**
- * gem_uses_aliasing_ppgtt:
+ * gem_gtt_type:
  * @fd: open i915 drm file descriptor
  *
- * Feature test macro to check whether the kernel internally uses ppgtt to
- * execute batches. The /aliasing/ in the function name is a bit a misnomer,
- * this driver parameter is also true when full ppgtt address spaces are
- * available since for batchbuffer construction only ppgtt or global gtt is
- * relevant.
+ * Feature test macro to check what type of gtt is being used by the kernel:
+ * 0 - global gtt
+ * 1 - aliasing ppgtt
+ * 2 - full ppgtt, limited to 32bit address space
+ * 3 - full ppgtt, 64bit address space
  *
- * Returns: Whether batches are run through ppgtt.
+ * Returns: Type of gtt being used.
  */
-bool gem_uses_aliasing_ppgtt(int fd)
+int gem_gtt_type(int fd)
 {
 	struct drm_i915_getparam gp;
 	int val = 0;
@@ -871,6 +1064,35 @@ bool gem_uses_aliasing_ppgtt(int fd)
 	return val;
 }
 
+/**
+ * gem_uses_ppgtt:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to check whether the kernel internally uses ppgtt to
+ * execute batches. Note that this is also true when we're using full ppgtt.
+ *
+ * Returns: Whether batches are run through ppgtt.
+ */
+bool gem_uses_ppgtt(int fd)
+{
+	return gem_gtt_type(fd) > 0;
+}
+
+/**
+ * gem_uses_full_ppgtt:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to check whether the kernel internally uses full
+ * per-process gtt to execute batches. Note that this is also true when we're
+ * using full 64b ppgtt.
+ *
+ * Returns: Whether batches are run through full ppgtt.
+ */
+bool gem_uses_full_ppgtt(int fd)
+{
+	return gem_gtt_type(fd) > 1;
+}
+
 /**
  * gem_available_fences:
  * @fd: open i915 drm file descriptor
@@ -918,54 +1140,7 @@ bool gem_has_llc(int fd)
 	return has_llc;
 }
 
-/**
- * gem_get_num_rings:
- * @fd: open i915 drm file descriptor
- *
- * Feature test macro to query the number of available rings. This is useful in
- * test loops which need to step through all rings and similar logic.
- *
- * For more explicit tests of ring availability see gem_has_enable_ring() and
- * the ring specific versions like gem_has_bsd().
- *
- * Returns: The number of available rings.
- */
-int gem_get_num_rings(int fd)
-{
-	static int num_rings = -1;
-
-	if (num_rings < 0) {
-		num_rings = 1;	/* render ring is always available */
-
-		if (gem_has_bsd(fd))
-			num_rings++;
-		else
-			goto skip;
-
-		if (gem_has_blt(fd))
-			num_rings++;
-		else
-			goto skip;
-
-		if (gem_has_vebox(fd))
-			num_rings++;
-		else
-			goto skip;
-	}
-skip:
-	return num_rings;
-}
-
-/**
- * gem_has_enable_ring:
- * @fd: open i915 drm file descriptor
- * @param: ring flag bit as used in gem_execbuf()
- *
- * Feature test macro to query whether a specific ring is available.
- *
- * Returns: Whether the ring is available or not.
- */
-bool gem_has_enable_ring(int fd,int param)
+static bool has_param(int fd, int param)
 {
 	drm_i915_getparam_t gp;
 	int tmp = 0;
@@ -974,7 +1149,7 @@ bool gem_has_enable_ring(int fd,int param)
 	gp.value = &tmp;
 	gp.param = param;
 
-	if (drmIoctl(fd, DRM_IOCTL_I915_GETPARAM, &gp))
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp))
 		return false;
 
 	errno = 0;
@@ -985,8 +1160,7 @@ bool gem_has_enable_ring(int fd,int param)
  * gem_has_bsd:
  * @fd: open i915 drm file descriptor
  *
- * Feature test macro to query whether the BSD ring is available. This is simply
- * a specific version of gem_has_enable_ring() for the BSD ring.
+ * Feature test macro to query whether the BSD ring is available.
  *
  * Note that recent Bspec calls this the VCS ring for Video Command Submission.
  *
@@ -996,7 +1170,7 @@ bool gem_has_bsd(int fd)
 {
 	static int has_bsd = -1;
 	if (has_bsd < 0)
-		has_bsd = gem_has_enable_ring(fd,I915_PARAM_HAS_BSD);
+		has_bsd = has_param(fd, I915_PARAM_HAS_BSD);
 	return has_bsd;
 }
 
@@ -1004,8 +1178,7 @@ bool gem_has_bsd(int fd)
  * gem_has_blt:
  * @fd: open i915 drm file descriptor
  *
- * Feature test macro to query whether the blitter ring is available. This is simply
- * a specific version of gem_has_enable_ring() for the blitter ring.
+ * Feature test macro to query whether the blitter ring is available.
  *
  * Note that recent Bspec calls this the BCS ring for Blitter Command Submission.
  *
@@ -1015,7 +1188,7 @@ bool gem_has_blt(int fd)
 {
 	static int has_blt = -1;
 	if (has_blt < 0)
-		has_blt =  gem_has_enable_ring(fd,I915_PARAM_HAS_BLT);
+		has_blt =  has_param(fd, I915_PARAM_HAS_BLT);
 	return has_blt;
 }
 
@@ -1024,8 +1197,7 @@ bool gem_has_blt(int fd)
  * gem_has_vebox:
  * @fd: open i915 drm file descriptor
  *
- * Feature test macro to query whether the vebox ring is available. This is simply
- * a specific version of gem_has_enable_ring() for the vebox ring.
+ * Feature test macro to query whether the vebox ring is available.
  *
  * Note that recent Bspec calls this the VECS ring for Video Enhancement Command
  * Submission.
@@ -1036,7 +1208,7 @@ bool gem_has_vebox(int fd)
 {
 	static int has_vebox = -1;
 	if (has_vebox < 0)
-		has_vebox =  gem_has_enable_ring(fd,LOCAL_I915_PARAM_HAS_VEBOX);
+		has_vebox =  has_param(fd, LOCAL_I915_PARAM_HAS_VEBOX);
 	return has_vebox;
 }
 
@@ -1045,8 +1217,7 @@ bool gem_has_vebox(int fd)
  * gem_has_bsd2:
  * @fd: open i915 drm file descriptor
  *
- * Feature test macro to query whether the BSD2 ring is available. This is simply
- * a specific version of gem_has_enable_ring() for the BSD2 ring.
+ * Feature test macro to query whether the BSD2 ring is available.
  *
  * Note that recent Bspec calls this the VCS ring for Video Command Submission.
  *
@@ -1056,7 +1227,7 @@ bool gem_has_bsd2(int fd)
 {
 	static int has_bsd2 = -1;
 	if (has_bsd2 < 0)
-		has_bsd2 = gem_has_enable_ring(fd,LOCAL_I915_PARAM_HAS_BSD2);
+		has_bsd2 = has_param(fd, LOCAL_I915_PARAM_HAS_BSD2);
 	return has_bsd2;
 }
 /**
@@ -1133,6 +1304,54 @@ uint64_t gem_mappable_aperture_size(void)
 	return pci_dev->regions[bar].size;
 }
 
+/**
+ * gem_global_aperture_size:
+ *
+ * Feature test macro to query the kernel for the global gpu aperture size.
+ * This is the area available for the kernel to perform address translations.
+ *
+ * Returns: The mappable gtt address space size.
+ */
+uint64_t gem_global_aperture_size(int fd)
+{
+	struct drm_i915_gem_get_aperture aperture;
+
+	memset(&aperture, 0, sizeof(aperture));
+	aperture.aper_size = 256*1024*1024;
+	do_ioctl(fd, DRM_IOCTL_I915_GEM_GET_APERTURE, &aperture);
+
+	return aperture.aper_size;
+}
+
+#define LOCAL_I915_PARAM_HAS_EXEC_SOFTPIN 37
+/**
+ * gem_has_softpin:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to query whether the softpinning functionality is
+ * supported.
+ *
+ * Returns: Whether softpin support is available
+ */
+bool gem_has_softpin(int fd)
+{
+	static int has_softpin = -1;
+
+	if (has_softpin < 0) {
+		struct drm_i915_getparam gp;
+
+		memset(&gp, 0, sizeof(gp));
+		gp.param = LOCAL_I915_PARAM_HAS_EXEC_SOFTPIN;
+		gp.value = &has_softpin;
+
+		has_softpin = 0;
+		ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp));
+		errno = 0;
+	}
+
+	return has_softpin;
+}
+
 /**
  * gem_require_caching:
  * @fd: open i915 drm file descriptor
@@ -1142,50 +1361,70 @@ uint64_t gem_mappable_aperture_size(void)
  */
 void gem_require_caching(int fd)
 {
-	struct local_drm_i915_gem_caching arg;
-	int ret;
+	uint32_t handle;
 
-	memset(&arg, 0, sizeof(arg));
-	arg.handle = gem_create(fd, 4096);
-	igt_assert(arg.handle != 0);
+	handle = gem_create(fd, 4096);
+	gem_set_caching(fd, handle, 0);
+	gem_close(fd, handle);
 
-	arg.caching = 0;
-	ret = ioctl(fd, LOCAL_DRM_IOCTL_I915_GEM_SET_CACHEING, &arg);
-	gem_close(fd, arg.handle);
-
-	igt_require(ret == 0);
 	errno = 0;
 }
 
+bool gem_has_ring(int fd, unsigned ring)
+{
+	struct drm_i915_gem_execbuffer2 execbuf;
+	struct drm_i915_gem_exec_object2 exec;
+
+	/* silly ABI, the kernel thinks everyone who has BSD also has BSD2 */
+	if ((ring & ~(3<<13)) == I915_EXEC_BSD) {
+		if (ring & (3 << 13) && !gem_has_bsd2(fd))
+			return false;
+	}
+
+	memset(&exec, 0, sizeof(exec));
+	memset(&execbuf, 0, sizeof(execbuf));
+	execbuf.buffers_ptr = (uintptr_t)&exec;
+	execbuf.buffer_count = 1;
+	execbuf.flags = ring;
+	return __gem_execbuf(fd, &execbuf) == -ENOENT;
+}
+
 /**
  * gem_require_ring:
  * @fd: open i915 drm file descriptor
- * @ring_id: ring flag bit as used in gem_execbuf()
+ * @ring: ring flag bit as used in gem_execbuf()
  *
  * Feature test macro to query whether a specific ring is available.
- * In contrast to gem_has_enable_ring() this automagically skips if the ring
- * isn't available by calling igt_require().
+ * This automagically skips if the ring isn't available by
+ * calling igt_require().
  */
-void gem_require_ring(int fd, int ring_id)
+void gem_require_ring(int fd, unsigned ring)
 {
-	switch (ring_id) {
-	case I915_EXEC_RENDER:
-		return;
-	case I915_EXEC_BLT:
-		igt_require(gem_has_blt(fd));
-		return;
-	case I915_EXEC_BSD:
-		igt_require(gem_has_bsd(fd));
-		return;
-#ifdef I915_EXEC_VEBOX
-	case I915_EXEC_VEBOX:
-		igt_require(gem_has_vebox(fd));
-		return;
-#endif
-	default:
-		igt_assert(0);
-		return;
-	}
+	igt_require(gem_has_ring(fd, ring));
+}
+
+/**
+ * gem_has_mocs_registers:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to query whether the device has MOCS registers.
+ * These exist gen 9+.
+ */
+bool gem_has_mocs_registers(int fd)
+{
+	return intel_gen(intel_get_drm_devid(fd)) >= 9;
+}
+
+/**
+ * gem_require_mocs_registers:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to query whether the device has MOCS registers.
+ * These exist gen 9+.
+ */
+void gem_require_mocs_registers(int fd)
+{
+	igt_require(gem_has_mocs_registers(fd));
 }
 
 /* prime */
@@ -1215,6 +1454,31 @@ int prime_handle_to_fd(int fd, uint32_t handle)
 	return args.fd;
 }
 
+/**
+ * prime_handle_to_fd_for_mmap:
+ * @fd: open i915 drm file descriptor
+ * @handle: file-private gem buffer object handle
+ *
+ * Same as prime_handle_to_fd above but with DRM_RDWR capabilities, which can
+ * be useful for writing into the mmap'ed dma-buf file-descriptor.
+ *
+ * Returns: The created dma-buf fd handle or -1 if the ioctl fails.
+ */
+int prime_handle_to_fd_for_mmap(int fd, uint32_t handle)
+{
+	struct drm_prime_handle args;
+
+	memset(&args, 0, sizeof(args));
+	args.handle = handle;
+	args.flags = DRM_CLOEXEC | DRM_RDWR;
+	args.fd = -1;
+
+	if (igt_ioctl(fd, DRM_IOCTL_PRIME_HANDLE_TO_FD, &args) != 0)
+		return -1;
+
+	return args.fd;
+}
+
 /**
  * prime_fd_to_handle:
  * @fd: open i915 drm file descriptor
@@ -1261,6 +1525,38 @@ off_t prime_get_size(int dma_buf_fd)
 	return ret;
 }
 
+/**
+ * prime_sync_start
+ * @dma_buf_fd: dma-buf fd handle
+ */
+void prime_sync_start(int dma_buf_fd, bool write)
+{
+	struct local_dma_buf_sync sync_start;
+
+	memset(&sync_start, 0, sizeof(sync_start));
+	sync_start.flags = LOCAL_DMA_BUF_SYNC_START;
+	sync_start.flags |= LOCAL_DMA_BUF_SYNC_READ;
+	if (write)
+		sync_start.flags |= LOCAL_DMA_BUF_SYNC_WRITE;
+	do_ioctl(dma_buf_fd, LOCAL_DMA_BUF_IOCTL_SYNC, &sync_start);
+}
+
+/**
+ * prime_sync_end
+ * @dma_buf_fd: dma-buf fd handle
+ */
+void prime_sync_end(int dma_buf_fd, bool write)
+{
+	struct local_dma_buf_sync sync_end;
+
+	memset(&sync_end, 0, sizeof(sync_end));
+	sync_end.flags = LOCAL_DMA_BUF_SYNC_END;
+	sync_end.flags |= LOCAL_DMA_BUF_SYNC_READ;
+	if (write)
+		sync_end.flags |= LOCAL_DMA_BUF_SYNC_WRITE;
+	do_ioctl(dma_buf_fd, LOCAL_DMA_BUF_IOCTL_SYNC, &sync_end);
+}
+
 /**
  * igt_require_fb_modifiers:
  * @fd: Open DRM file descriptor.
@@ -1303,7 +1599,7 @@ int __kms_addfb(int fd, uint32_t handle, uint32_t width, uint32_t height,
 	f.pitches[0] = stride;
 	f.modifier[0] = modifier;
 
-	ret = drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f);
+	ret = igt_ioctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f);
 
 	*buf_id = f.fb_id;
 
