@@ -130,11 +130,9 @@ copy(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo, int n_bo)
 	return ret;
 }
 
-static void clear(int fd, uint32_t handle, int size)
+static void clear(int fd, uint32_t handle, uint64_t size)
 {
-	void *base = gem_mmap__cpu(fd, handle, 0, size, PROT_READ | PROT_WRITE);
-
-	igt_assert(base != NULL);
+	void *base = gem_mmap__cpu(fd, handle, 0, size, PROT_WRITE);
 	memset(base, 0, size);
 	munmap(base, size);
 }
@@ -146,10 +144,10 @@ static struct igt_eviction_test_ops fault_ops = {
 	.clear = clear,
 };
 
-static void test_forking_evictions(int fd, int size, int count,
-					unsigned flags)
+static void test_forking_evictions(int fd, uint64_t size, uint64_t count,
+				   unsigned flags)
 {
-	int trash_count;
+	uint64_t trash_count;
 
 	trash_count = intel_get_total_ram_mb() * 11 / 10;
 	intel_require_memory(trash_count, size, CHECK_RAM | CHECK_SWAP);
@@ -157,12 +155,12 @@ static void test_forking_evictions(int fd, int size, int count,
 	forking_evictions(fd, &fault_ops, size, count, trash_count, flags);
 }
 
-static void test_mlocked_evictions(int fd, int size, int count)
+static void test_mlocked_evictions(int fd, uint64_t size, uint64_t count)
 {
 	mlocked_evictions(fd, &fault_ops, size, count);
 }
 
-static void test_swapping_evictions(int fd, int size, int count)
+static void test_swapping_evictions(int fd, uint64_t size, uint64_t count)
 {
 	int trash_count;
 
@@ -172,19 +170,21 @@ static void test_swapping_evictions(int fd, int size, int count)
 	swapping_evictions(fd, &fault_ops, size, count, trash_count);
 }
 
-static void test_minor_evictions(int fd, int size, int count)
+static void test_minor_evictions(int fd, uint64_t size, uint64_t count)
 {
 	minor_evictions(fd, &fault_ops, size, count);
 }
 
-static void test_major_evictions(int fd, int size, int count)
+static void test_major_evictions(int fd, uint64_t size, uint64_t count)
 {
 	major_evictions(fd, &fault_ops, size, count);
 }
 
 igt_main
 {
-	int size, count, fd;
+	uint64_t size, count;
+	int fd;
+
 	size = count = 0;
 	fd = -1;
 
@@ -244,32 +244,32 @@ igt_main
 		test_major_evictions(fd, size, count);
 	}
 
-	if (igt_fork_hang_helper()) {
-		igt_fixture {
-			size = 1024 * 1024;
-			count = 3*gem_aperture_size(fd) / size / 4;
-		}
+	igt_fixture {
+		igt_fork_hang_helper();
 
-		igt_subtest("mlocked-hang")
-			test_mlocked_evictions(fd, size, count);
-
-		igt_subtest("swapping-hang")
-			test_swapping_evictions(fd, size, count);
-
-		igt_subtest("minor-hang")
-			test_minor_evictions(fd, size, count);
-
-		igt_subtest("major-hang") {
-			size = 3*gem_aperture_size(fd) / 4;
-			count = 4;
-			test_major_evictions(fd, size, count);
-		}
-
-		igt_stop_hang_helper();
+		size = 1024 * 1024;
+		count = 3*gem_aperture_size(fd) / size / 4;
 	}
-	igt_stop_signal_helper();
+
+	igt_subtest("mlocked-hang")
+		test_mlocked_evictions(fd, size, count);
+
+	igt_subtest("swapping-hang")
+		test_swapping_evictions(fd, size, count);
+
+	igt_subtest("minor-hang")
+		test_minor_evictions(fd, size, count);
+
+	igt_subtest("major-hang") {
+		size = 3*gem_aperture_size(fd) / 4;
+		count = 4;
+		test_major_evictions(fd, size, count);
+	}
+
+	igt_stop_hang_helper();
 
 	igt_fixture {
+		igt_stop_signal_helper();
 		close(fd);
 	}
 }
