@@ -130,7 +130,7 @@ static int do_writeval(FILE *filp, int val, int lerrno, bool readback_check)
 			igt_assert_eq(readval(filp), orig);
 	} else {
 		/* Expecting no error */
-		igt_assert_neq(ret, 0);
+		igt_assert_lt(0, ret);
 		wait_freq_settle();
 		if (readback_check)
 			igt_assert_eq(readval(filp), val);
@@ -146,7 +146,7 @@ static void checkit(const int *freqs)
 {
 	igt_assert_lte(freqs[MIN], freqs[MAX]);
 	igt_assert_lte(freqs[CUR], freqs[MAX]);
-	igt_assert_lte(freqs[MIN], freqs[CUR]);
+	igt_assert_lte(freqs[RPn], freqs[CUR]);
 	igt_assert_lte(freqs[RPn], freqs[MIN]);
 	igt_assert_lte(freqs[MAX], freqs[RP0]);
 	igt_assert_lte(freqs[RP1], freqs[RP0]);
@@ -288,7 +288,7 @@ static void load_helper_stop(void)
 static void load_helper_init(void)
 {
 	lh.devid = intel_get_drm_devid(drm_fd);
-	lh.has_ppgtt = gem_uses_aliasing_ppgtt(drm_fd);
+	lh.has_ppgtt = gem_uses_ppgtt(drm_fd);
 
 	/* MI_STORE_DATA can only use GTT address on gen4+/g33 and needs
 	 * snoopable mem on pre-gen6. Hence load-helper only works on gen6+, but
@@ -388,10 +388,14 @@ static void min_max_config(void (*check)(void), bool load_gpu)
 
 	igt_debug("\nIncrease min to midpoint...\n");
 	writeval(stuff[MIN].filp, fmid);
+	if (load_gpu)
+		do_load_gpu();
 	check();
 
 	igt_debug("\nIncrease min to RP0...\n");
 	writeval(stuff[MIN].filp, origfreqs[RP0]);
+	if (load_gpu)
+		do_load_gpu();
 	check();
 
 	igt_debug("\nIncrease min above RP0 (invalid)...\n");
@@ -460,7 +464,7 @@ static void basic_check(void)
 }
 
 #define IDLE_WAIT_TIMESTEP_MSEC 100
-#define IDLE_WAIT_TIMEOUT_MSEC 10000
+#define IDLE_WAIT_TIMEOUT_MSEC 15000
 static void idle_check(void)
 {
 	int freqs[NUMFREQ];
@@ -472,14 +476,14 @@ static void idle_check(void)
 		read_freqs(freqs);
 		dump(freqs);
 		checkit(freqs);
-		if (freqs[CUR] == freqs[MIN])
+		if (freqs[CUR] == freqs[RPn])
 			break;
 		usleep(1000 * IDLE_WAIT_TIMESTEP_MSEC);
 		wait += IDLE_WAIT_TIMESTEP_MSEC;
 	} while (wait < IDLE_WAIT_TIMEOUT_MSEC);
 
-	igt_assert_eq(freqs[CUR], freqs[MIN]);
-	igt_debug("Required %d msec to reach cur=min\n", wait);
+	igt_assert_eq(freqs[CUR], freqs[RPn]);
+	igt_debug("Required %d msec to reach cur=idle\n", wait);
 }
 
 #define LOADED_WAIT_TIMESTEP_MSEC 100
@@ -506,7 +510,7 @@ static void loaded_check(void)
 }
 
 #define STABILIZE_WAIT_TIMESTEP_MSEC 100
-#define STABILIZE_WAIT_TIMEOUT_MSEC 10000
+#define STABILIZE_WAIT_TIMEOUT_MSEC 15000
 static void stabilize_check(int *freqs)
 {
 	int wait = 0;
@@ -521,6 +525,13 @@ static void stabilize_check(int *freqs)
 	igt_debug("Waited %d msec to stabilize cur\n", wait);
 }
 
+static void reset_gpu(void)
+{
+	int fd = drm_open_driver(DRIVER_INTEL);
+	igt_post_hang_ring(fd, igt_hang_ring(fd, I915_EXEC_DEFAULT));
+	close(fd);
+}
+
 /*
  * reset - test that turbo works across a ring stop
  *
@@ -556,11 +567,8 @@ static void reset(void)
 	load_helper_run(LOW);
 	stabilize_check(pre_freqs);
 
-	igt_debug("Stop rings...\n");
-	igt_set_stop_rings(STOP_RING_DEFAULTS);
-	while (igt_get_stop_rings())
-		usleep(1000 * 100);
-	igt_debug("Ring stop cleared\n");
+	igt_debug("Reset gpu...\n");
+	reset_gpu();
 
 	igt_debug("Apply high load...\n");
 	load_helper_set_load(HIGH);
