@@ -132,7 +132,8 @@ static void dump_connectors_fd(int drmfd)
 	for (i = 0; i < mode_resources->count_connectors; i++) {
 		drmModeConnector *connector;
 
-		connector = drmModeGetConnector(drmfd, mode_resources->connectors[i]);
+		connector = drmModeGetConnectorCurrent(drmfd,
+						       mode_resources->connectors[i]);
 		if (!connector) {
 			igt_warn("could not get connector %i: %s\n", mode_resources->connectors[i], strerror(errno));
 			continue;
@@ -190,12 +191,19 @@ static void dump_info(void)
 
 static void connector_find_preferred_mode(uint32_t connector_id,
 					  unsigned long crtc_idx_mask,
-					  int mode_num, struct connector *c)
+					  int mode_num, struct connector *c,
+					  bool probe)
 {
 	struct kmstest_connector_config config;
+	bool ret;
 
-	if (!kmstest_get_connector_config(drm_fd, connector_id, crtc_idx_mask,
-					  &config)) {
+	if (probe)
+		ret = kmstest_probe_connector_config(drm_fd, connector_id,
+						     crtc_idx_mask, &config);
+	else
+		ret = kmstest_get_connector_config(drm_fd, connector_id,
+						   crtc_idx_mask, &config);
+	if (!ret) {
 		c->mode_valid = 0;
 		return;
 	}
@@ -218,21 +226,15 @@ static void connector_find_preferred_mode(uint32_t connector_id,
 static void
 paint_color_key(struct igt_fb *fb_info)
 {
-	int i, j;
-	uint32_t *fb_ptr;
+	cairo_t *cr = igt_get_cairo_ctx(drm_fd, fb_info);
 
-	fb_ptr = gem_mmap__gtt(drm_fd, fb_info->gem_handle, fb_info->size,
-			       PROT_READ | PROT_WRITE);
+	cairo_rectangle(cr, crtc_x, crtc_y, crtc_w, crtc_h);
+	cairo_set_source_rgb(cr, .8, .8, .8);
+	cairo_fill(cr);
 
-	for (i = crtc_y; i < crtc_y + crtc_h; i++)
-		for (j = crtc_x; j < crtc_x + crtc_w; j++) {
-			uint32_t offset;
+	igt_assert(!cairo_status(cr));
 
-			offset = (i * fb_info->stride / 4) + j;
-			fb_ptr[offset] = SPRITE_COLOR_KEY;
-		}
-
-	munmap(fb_ptr, fb_info->size);
+	cairo_destroy(cr);
 }
 
 static void paint_image(cairo_t *cr, const char *file)
@@ -258,8 +260,6 @@ static void paint_output_info(struct connector *c, struct igt_fb *fb)
 	double max_width;
 	int i;
 
-	igt_paint_test_pattern(cr, l_width, l_height);
-
 	cairo_move_to(cr, l_width / 2, l_height / 2);
 
 	/* Print connector and mode name */
@@ -352,9 +352,9 @@ set_mode(struct connector *c)
 		width = c->mode.hdisplay;
 		height = c->mode.vdisplay;
 
-		fb_id = igt_create_fb(drm_fd, width, height,
-				      igt_bpp_depth_to_drm_format(bpp, depth),
-				      tiling, &fb_info[current_fb]);
+		fb_id = igt_create_pattern_fb(drm_fd, width, height,
+					      igt_bpp_depth_to_drm_format(bpp, depth),
+					      tiling, &fb_info[current_fb]);
 		paint_output_info(c, &fb_info[current_fb]);
 		paint_color_key(&fb_info[current_fb]);
 
@@ -457,7 +457,7 @@ set_stereo_mode(struct connector *c)
  * Each connector has a corresponding encoder, except in the SDVO case
  * where an encoder may have multiple connectors.
  */
-int update_display(void)
+int update_display(bool probe)
 {
 	struct connector *connectors;
 	int c;
@@ -489,7 +489,7 @@ int update_display(void)
 			connector_find_preferred_mode(connector->id,
 						      crtc_idx_mask,
 						      specified_mode_num,
-						      connector);
+						      connector, probe);
 			if (!connector->mode_valid)
 				continue;
 
@@ -514,7 +514,7 @@ int update_display(void)
 			connector_find_preferred_mode(connector->id,
 						      -1UL,
 						      specified_mode_num,
-						      connector);
+						      connector, probe);
 			if (!connector->mode_valid)
 				continue;
 
@@ -724,7 +724,7 @@ int main(int argc, char **argv)
 	    specified_mode_num == -1 && !test_stereo_modes)
 		test_all_modes = 1;
 
-	drm_fd = drm_open_driver(DRIVER_INTEL);
+	drm_fd = drm_open_driver(DRIVER_ANY);
 
 	if (test_stereo_modes &&
 	    drmSetClientCap(drm_fd, DRM_CLIENT_CAP_STEREO_3D, 1) < 0) {
@@ -766,7 +766,7 @@ int main(int argc, char **argv)
 
 	ret = 0;
 
-	if (!update_display()) {
+	if (!update_display(false)) {
 		ret = 1;
 		goto out_stdio;
 	}
