@@ -28,7 +28,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/io.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -38,6 +37,22 @@
 
 #include "intel_reg_spec.h"
 
+
+#ifdef HAVE_SYS_IO_H
+#include <sys/io.h>
+#else
+
+static inline int _not_supported(void)
+{
+       fprintf(stderr, "portio-vga not supported\n");
+       exit(EXIT_FAILURE);
+}
+#define inb(port)              _not_supported()
+#define outb(value, port)      _not_supported()
+#define iopl(level)
+
+#endif /* HAVE_SYS_IO_H */
+
 struct config {
 	struct pci_device *pci_dev;
 	char *mmiofile;
@@ -424,7 +439,7 @@ static int intel_reg_write(struct config *config, int argc, char *argv[])
 		return EXIT_FAILURE;
 	}
 
-	intel_register_access_init(intel_get_pci_device(), 0);
+	intel_register_access_init(config->pci_dev, 0);
 
 	for (i = 1; i < argc; i += 2) {
 		struct reg reg;
@@ -491,7 +506,8 @@ static int intel_reg_snapshot(struct config *config, int argc, char *argv[])
 	intel_mmio_use_pci_bar(config->pci_dev);
 
 	/* XXX: error handling */
-	write(1, igt_global_mmio, config->pci_dev->regions[mmio_bar].size);
+	if (write(1, igt_global_mmio, config->pci_dev->regions[mmio_bar].size) == -1)
+		fprintf(stderr, "Error writing snapshot: %s", strerror(errno));
 
 	if (config->verbosity > 0)
 		printf("use this with --mmio=FILE --devid=0x%04X\n",
@@ -614,6 +630,11 @@ static int intel_reg_help(struct config *config, int argc, char *argv[])
 	printf("REGISTER is defined as:\n");
         printf("  [(PORTNAME|PORTNUM|MMIO-OFFSET):](REGNAME|REGADDR)\n");
 
+	printf("\n");
+	printf("PORTNAME is one of:\n");
+	intel_reg_spec_print_ports();
+	printf("\n");
+
 	printf("\n");
 	printf("OPTIONS common to most COMMANDS:\n");
 	printf(" --spec=PATH    Read register spec from directory or file\n");
@@ -648,6 +669,8 @@ static const char *get_codename(uint32_t devid)
 		return "broadwell";
 	else if (IS_SKYLAKE(devid))
 		return "skylake";
+	else if (IS_KABYLAKE(devid))
+		return "kabylake";
 	else if (IS_CHERRYVIEW(devid))
 		return "cherryview";
 	else if (IS_VALLEYVIEW(devid))
@@ -698,7 +721,7 @@ static int get_reg_spec_file(char *buf, size_t buflen, const char *dir,
 static int read_reg_spec(struct config *config)
 {
 	char buf[PATH_MAX];
-	char *path;
+	const char *path;
 	struct stat st;
 	int r;
 
@@ -707,7 +730,7 @@ static int read_reg_spec(struct config *config)
 		path = getenv("INTEL_REG_SPEC");
 
 	if (!path)
-		goto builtin;
+		path = PKGDATADIR"/registers";
 
 	r = stat(path, &st);
 	if (r) {
@@ -850,7 +873,7 @@ int main(int argc, char *argv[])
 	argc -= optind;
 	argv += optind;
 
-	if (help)
+	if (help || (argc > 0 && strcmp(argv[0], "help") == 0))
 		return intel_reg_help(&config, argc, argv);
 
 	if (argc == 0) {
