@@ -3,7 +3,7 @@
  * Copyright 2006 Tungsten Graphics, Inc., Cedar Park, Texas.
  * All Rights Reserved.
  *
- * Copyright 2014 Intel Corporation
+ * Copyright 2014, 2015 Intel Corporation
  * All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -164,7 +164,7 @@ unsigned intel_batch_num_cmds(struct intel_batchbuffer *batch)
 	return bb_area_items(batch->cmds);
 }
 
-static unsigned intel_batch_num_state(struct intel_batchbuffer *batch)
+unsigned intel_batch_num_state(struct intel_batchbuffer *batch)
 {
 	return bb_area_items(batch->state);
 }
@@ -202,7 +202,7 @@ uint32_t intel_batch_state_alloc(struct intel_batchbuffer *batch, unsigned bytes
 }
 
 uint32_t intel_batch_state_copy(struct intel_batchbuffer *batch,
-				void *d, unsigned bytes,
+				const void *d, unsigned bytes,
 				unsigned align,
 				const char *str)
 {
@@ -217,9 +217,10 @@ uint32_t intel_batch_state_copy(struct intel_batchbuffer *batch,
 
 	for (i = 0; i < dwords; i++) {
 		char offsetinside[80];
+		const uint32_t *s;
 		sprintf(offsetinside, "%s: 0x%x", str, i * 4);
 
-		uint32_t *s = (uint32_t *)(uint8_t *)d + i;
+		s = (const uint32_t *)(const uint8_t *)d + i;
 		bb_area_emit(batch->state, *s, STATE, offsetinside);
 	}
 
@@ -274,3 +275,18 @@ const char *intel_batch_type_as_str(const struct bb_item *item)
 
 	return "UNKNOWN";
 }
+
+void intel_batch_cmd_emit_null(struct intel_batchbuffer *batch,
+			       const int cmd, const int len, const int len_bias,
+			       const char *str)
+{
+	int i;
+
+	assert(len > 1);
+	assert((len - len_bias) >= 0);
+
+	bb_area_emit(batch->cmds, (cmd | (len - len_bias)), CMD, str);
+
+	for (i = len_bias-1; i < len; i++)
+		OUT_BATCH(0);
+}
